@@ -8,7 +8,7 @@
  * published by the Free Software Foundation; either version 2 of
  * the License or (at your option) version 3 or any later version
  * accepted by the membership of KDE e.V. (or its successor approved
- * by the membership of KDE e.V.), which shall act as a proxy 
+ * by the membership of KDE e.V.), which shall act as a proxy
  * defined in Section 14 of version 3 of the license.
  *
  * This program is distributed in the hope that it will be useful,
@@ -20,15 +20,15 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  ***********************************************************************/
 
+#include <config.h>
 
-#define CODEINE_DEBUG_PREFIX "engine"
+#define DRAGONPLAYER_DEBUG_PREFIX "engine"
 
 #include "videoWindow.h"
 #include "timeLabel.h"
 
 #include "actions.h"        //::seek() FIXME unfortunate
 #include "debug.h"
-#include "mxcl.library.h"
 #include "theStream.h"
 
 #ifdef HAVE_XINE
@@ -40,8 +40,10 @@
 #include <QLabel>
 #include <QTimer>
 #include <QVBoxLayout>
+#include <QPainter>
 
 #include <KApplication>
+#include <KIcon>
 #include <KLocale>
 #include <KMenu>
 #include <KMimeType>
@@ -65,6 +67,10 @@
 #include <windows.h>
 #endif
 
+#if defined(HAVE_UNISTD_H)
+#include <unistd.h>
+#endif
+
 using Phonon::AudioOutput;
 using Phonon::MediaObject;
 using Phonon::VideoWidget;
@@ -72,7 +78,7 @@ using Phonon::SeekSlider;
 using Phonon::VolumeSlider;
 using Phonon::MediaController;
 
-namespace Codeine {
+namespace Dragon {
 
 
 VideoWindow *VideoWindow::s_instance = 0;
@@ -81,13 +87,15 @@ VideoWindow::VideoWindow( QWidget *parent )
         : QWidget( parent )
         , m_cursorTimer( new QTimer( this ) )
         , m_justLoaded( false )
-		, m_adjustedSize( false)
+        , m_adjustedSize( false)
         , m_xineStream( 0 )
         , m_subLanguages( new QActionGroup( this ) )
         , m_audioLanguages( new QActionGroup( this ) )
         , m_logo( new QLabel( this ) )
 {
     DEBUG_BLOCK
+    
+    m_isPreview = false;
 
     s_instance = this;
     setObjectName( "VideoWindow" );
@@ -105,10 +113,12 @@ VideoWindow::VideoWindow( QWidget *parent )
     m_audioPath = Phonon::createPath(m_media, m_aOutput);
     m_media->setTickInterval( 1000 );
     connect( m_media, SIGNAL( tick( qint64 ) ), this, SIGNAL( tick( qint64 ) ) );
+    connect( m_media, SIGNAL( currentSourceChanged( Phonon::MediaSource ) ), this, SIGNAL( currentSourceChanged( Phonon::MediaSource ) ) );
     connect( m_media, SIGNAL( totalTimeChanged( qint64 ) ), this, SIGNAL( totalTimeChanged( qint64 ) ) );
     connect( m_media, SIGNAL( seekableChanged( bool ) ), this, SIGNAL( seekableChanged( bool ) ) );
+    connect( m_media, SIGNAL( metaDataChanged() ), this, SIGNAL( metaDataChanged() ) );
     connect( m_aOutput, SIGNAL( mutedChanged( bool ) ), this, SIGNAL( mutedChanged( bool ) ) );
-
+    connect( m_media, SIGNAL( hasVideoChanged( bool ) ), this, SIGNAL( hasVideoChanged( bool ) ) );
     connect( m_media, SIGNAL( hasVideoChanged( bool ) ), m_vWidget, SLOT( setVisible( bool ) ) ); //hide video widget if no video to show
     connect( m_media, SIGNAL( hasVideoChanged( bool ) ), m_logo, SLOT( setHidden( bool ) ) );
 
@@ -145,15 +155,12 @@ VideoWindow::VideoWindow( QWidget *parent )
         m_logo->setPalette( pal );
         QLayout* layout = new QVBoxLayout( m_logo );
         layout->setAlignment( Qt::AlignCenter );
-//        QLabel* logoImage = new QLabel( m_logo );
-//        logoImage->setPixmap( KStandardDirs::locate( "appdata",  "dragonlogo.png" ) );
-//        layout->addWidget( logoImage );
         m_logo->setLayout( layout );
         box->addWidget( m_logo );
         m_logo->show();
     }
     {
-        KConfigGroup config = KGlobal::config()->group( "General" ); 
+        KConfigGroup config = KGlobal::config()->group( "General" );
         m_aOutput->setVolume( config.readEntry<double>( "Volume", 1.0 ) );
     }
 }
@@ -161,15 +168,6 @@ VideoWindow::VideoWindow( QWidget *parent )
 VideoWindow::~VideoWindow()
 {
     DEBUG_BLOCK
-    /* let us never forget, mxcl's cum variable. RIP
-    int cum = 0;
-    for( int v = 99; v >= 0; v-- ) {
-        xine_set_param( m_stream, XINE_PARAM_AUDIO_AMP_LEVEL, v );
-        int sleep = int(32000 * (-std::log10( double(v + 1) ) + 2));
-        ::usleep( sleep );
-        cum += sleep;
-    }
-    debug() << "Total sleep: " << cum << "x10^-6 s\n"; */
 
     eject();
     KConfigGroup config = KGlobal::config()->group( "General" );
@@ -201,7 +199,9 @@ bool
 VideoWindow::load( const KUrl &url )
 {
     DEBUG_BLOCK
-    mxcl::WaitCursor allocateOnStack;
+
+    QApplication::setOverrideCursor( Qt::WaitCursor );
+
     eject();
 
     KMimeType::Ptr mimeType = KMimeType::findByUrl( url );
@@ -212,6 +212,10 @@ VideoWindow::load( const KUrl &url )
         m_media->setCurrentSource( url );
     m_justLoaded = true;
     m_adjustedSize=false;
+    engine()->play();
+
+    QApplication::restoreOverrideCursor();
+
     return true;
 }
 
@@ -219,11 +223,17 @@ bool
 VideoWindow::play( qint64 offset )
 {
     DEBUG_BLOCK
-    mxcl::WaitCursor allocateOnStack;
+
+    QApplication::setOverrideCursor( Qt::WaitCursor );
+
     m_justLoaded = false;
     if( offset > 0 )
         seek( offset );
     m_media->play();
+    debug() << "Does this media have Video stream? " << TheStream::hasVideo();
+
+    QApplication::restoreOverrideCursor();
+
     return true;
 }
 
@@ -296,23 +306,42 @@ DEBUG_BLOCK
     }
 }
 
+bool 
+VideoWindow::isPreview(const bool &v)
+{
+   if( v )
+     {
+       m_isPreview = v;
+     }
+   return m_isPreview;
+}
+
 void
 VideoWindow::relativeSeek( qint64 step )
 {
-    m_media->pause();
+    debug() << "** relative seek";
     const qint64 new_pos = currentTime() + step;
-    if( new_pos > 0 )
+    if( ( new_pos >= 0 ) && ( new_pos < length() ) )
+    {
         seek( new_pos );
-    play();
+        play();
+    }
+    else if( new_pos < 0 )
+    {
+        seek( 0 );
+        play();
+    }
 }
 
 void
 VideoWindow::stop()
 {
+    debug() << "Stop called";
     eject();
     m_media->stop();
-    m_media->setCurrentSource( Phonon::MediaSource() ); //set the current source to invalid
-    m_vWidget->hide();
+    m_media->setCurrentSource(Phonon::MediaSource()); //set the current source to    Phonon::MediaSource::Empty
+    debug() << "Media source valid? "<<  TheStream::hasMedia();
+      m_vWidget->hide();
     m_logo->show();
 }
 
@@ -322,6 +351,15 @@ VideoWindow::pause()
     m_media->pause();
 }
 
+void
+VideoWindow::playPause()
+{
+  if(m_media->state() == Phonon::PlayingState)
+    pause();
+  else
+    resume();
+}
+
 QString
 VideoWindow::urlOrDisc() const
 {
@@ -329,6 +367,7 @@ VideoWindow::urlOrDisc() const
     switch( source.type() )
     {
         case Phonon::MediaSource::Invalid:
+        case Phonon::MediaSource::Empty:
             return "Invalid"; //no i18n, used for DBus responses
             break;
         case Phonon::MediaSource::Url:
@@ -347,7 +386,7 @@ VideoWindow::urlOrDisc() const
     return "Error";
 }
 
-QMultiMap<QString, QString> 
+QMultiMap<QString, QString>
 VideoWindow::metaData() const
 {
     return m_media->metaData();
@@ -360,50 +399,10 @@ VideoWindow::isSeekable() const
 }
 
 
-Engine::State
+Phonon::State
 VideoWindow::state() const
 {
-    return state( m_media->state() ); 
-}
-
-Engine::State
-VideoWindow::state( Phonon::State state ) const
-{
-    if( m_media->currentSource().type() == Phonon::MediaSource::Invalid )
-        return Engine::Empty;
-    else if( m_justLoaded )
-        return Engine::Loaded;
-    
-/*
-    enum State
-    {
-        Uninitialised = 0,
-        Empty = 1,
-        Loaded = 2,
-        Playing = 4,
-        Paused = 8,
-        TrackEnded = 16
-    }; */
-    switch( state )
-    {
-        case Phonon::StoppedState:
-            return Engine::TrackEnded;
-        break;
-        case Phonon::BufferingState:
-        case Phonon::LoadingState:
-            return Engine::Loaded;
-		break;
-        case Phonon::PlayingState:
-            return Engine::Playing;
-        break;
-        case Phonon::PausedState:
-            return Engine::Paused;
-        break;
-        case Phonon::ErrorState:
-        default:
-            return Engine::Uninitialised;
-        break;
-    }
+    return m_media->state();
 }
 
 qreal
@@ -443,19 +442,8 @@ VideoWindow::seek( qint64 pos )
     // stopped but xine is actually playing the track. Tada!
     // TODO set state based on events from xine only
 
-    switch( state() ) {
-    case Engine::Uninitialised:
-        //NOTE should never happen
-        Debug::warning() << "Seek attempt thwarted! xine not initialised!\n";
-        return;
-    case Engine::Empty:
-        Debug::warning() << "Seek attempt thwarted! No media loaded!\n";
-        return;
-    default:
-        ;
-    }
-    m_media->pause(); //pausing first gives Phonon a chance to recognize seekable media
-    m_media->seek( pos );
+      m_media->pause(); //pausing first gives Phonon a chance to recognize seekable media;
+      m_media->seek( pos );
 }
 
 void
@@ -494,6 +482,7 @@ VideoWindow::newPositionSlider()
 {
     SeekSlider *seekSlider = new SeekSlider();
     seekSlider->setMediaObject( m_media );
+    seekSlider->setSingleStep( 5000 );
     return seekSlider;
 }
 
@@ -535,11 +524,9 @@ debug() << "chapters: " << m_controller->availableChapters() << " titles: " << m
     debug() << "going from " << states.at(oldstate) << " to " << states.at(currentState);
 
     if( currentState == Phonon::LoadingState )
-        m_xineStream = 0;
+      m_xineStream = 0;
 
-
-    //N.B this code is also run when coming out of Paused state, as Phonon goes Paused->Buffering->Playing (but at least this saves doing it twice)
-    if( currentState == Phonon::PlayingState && oldstate != Phonon::PausedState && m_media->hasVideo() )
+    if( currentState == Phonon::PlayingState  && m_media->hasVideo() )
     {
         m_logo->hide();
         m_vWidget->show();
@@ -552,9 +539,9 @@ debug() << "chapters: " << m_controller->availableChapters() << " titles: " << m
              ( (QWidget*) mainWindow() )->adjustSize();
           m_adjustedSize=true;
           debug() << "adjusting size to video resolution";
-        }     
+        }
     }
-    emit stateChanged( state( currentState ) ); 
+    emit stateUpdated( currentState, oldstate );
 }
 
 void
@@ -648,7 +635,8 @@ VideoWindow::setSubtitle( int channel )
     DEBUG_BLOCK
     Phonon::SubtitleDescription desc = Phonon::SubtitleDescription::fromIndex( channel );
     debug() << "using index: " << channel << " returned desc has index: " << desc.index();
-    m_controller->setCurrentSubtitle( desc );
+    if(desc.isValid())
+      m_controller->setCurrentSubtitle( desc );
 }
 
 void
@@ -665,7 +653,8 @@ VideoWindow::setAudioChannel( int channel )
     DEBUG_BLOCK
     Phonon::AudioChannelDescription desc = Phonon::AudioChannelDescription::fromIndex( channel );
     debug() << "using index: " << channel << " returned desc has index: " << desc.index();
-    m_controller->setCurrentAudioChannel( desc );
+    if(desc.isValid())
+      m_controller->setCurrentAudioChannel( desc );
 }
 
 void
@@ -747,19 +736,15 @@ VideoWindow::tenPercentForward()
 void
 VideoWindow::tenSecondsBack()
 {
-  qint64 newTime = m_media->currentTime() - 10;
-  if (newTime > 0)
-    m_media -> seek (newTime);
-  else
-    m_media -> seek ( 0 ) ;
+    qint64 newTime = m_media->currentTime() - 10000;
+    relativeSeek( newTime );
 }
 
 void
 VideoWindow::tenSecondsForward()
 {
-  qint64 newTime = m_media->currentTime() + 10;
-  if (newTime < m_media->totalTime())
-    m_media -> seek (newTime);
+    qint64 newTime = m_media->currentTime() + 10000;
+    relativeSeek( newTime );
 }
 
 ///////////
@@ -784,8 +769,6 @@ VideoWindow::event( QEvent* event )
       case QEvent::MouseButtonPress:
          kapp->restoreOverrideCursor();
          if( hasFocus() )
-            // see above comment
-            debug() << "cursor will disappear in 2000 seconds";
             m_cursorTimer->start( CURSOR_HIDE_TIMEOUT );
          break;
       default: return QWidget::event( event );
@@ -803,7 +786,7 @@ VideoWindow::contextMenuEvent( QContextMenuEvent * event )
         menu.addAction( action( "play" ) );
         menu.addAction( action( "fullscreen" ) );
         menu.addAction( action( "reset_zoom" ) );
-        if(isDVD()) 
+        if(isDVD())
         {
             menu.addAction( action( "toggle_dvd_menu" ) );
         }
@@ -811,7 +794,7 @@ VideoWindow::contextMenuEvent( QContextMenuEvent * event )
     menu.exec( event->globalPos() );
 }
 
-void 
+void
 VideoWindow::mouseDoubleClickEvent( QMouseEvent* )
 {
     if( mainWindow() ) //TODO: add full screen mode to kpart
@@ -842,6 +825,10 @@ DEBUG_BLOCK
     if( m_media->currentSource().type() == Phonon::MediaSource::Invalid )
         return;
 
+    if( m_media->currentSource().type() == Phonon::MediaSource::Empty )
+        return;
+
+
     KConfigGroup profile = TheStream::profile(); // the config profile for this video file
 
     Phonon::State state = m_media->state();
@@ -888,6 +875,6 @@ DEBUG_BLOCK
     profile.sync();
 }
 
-} //namespace Codeine
+} //namespace Dragon
 
 #include "videoWindow.moc"
