@@ -13,13 +13,14 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "tagrenameroptions.h"
+
 #include <kdebug.h>
 #include <kglobal.h>
 #include <klocale.h>
 #include <kconfig.h>
 #include <kconfigbase.h>
-
-#include "tagrenameroptions.h"
+#include <kconfiggroup.h>
 
 TagRenamerOptions::TagRenamerOptions() :
     m_emptyAction(IgnoreEmptyTag),
@@ -51,6 +52,8 @@ TagRenamerOptions::TagRenamerOptions(const CategoryID &category)
     switch(category.category) {
     case Title:
     case Artist:
+    case Genre:
+    case Year:
     case Album:
     case Track:
         disabled = false;
@@ -67,12 +70,12 @@ TagRenamerOptions::TagRenamerOptions(const CategoryID &category)
     if(categoryNum > 0)
         typeKey.append(QString::number(categoryNum));
 
-    setSuffix(config.readEntry(QString("%1Suffix").arg(typeKey)));
-    setPrefix(config.readEntry(QString("%1Prefix").arg(typeKey)));
+    setSuffix(config.readEntry(QString("%1Suffix").arg(typeKey), QString()));
+    setPrefix(config.readEntry(QString("%1Prefix").arg(typeKey), QString()));
 
     // Default the emptyAction to ignoring the empty tag.
 
-    const QString emptyAction = config.readEntry(QString("%1EmptyAction").arg(typeKey)).lower();
+    const QString emptyAction = config.readEntry(QString("%1EmptyAction").arg(typeKey), QString()).toLower();
     setEmptyAction(IgnoreEmptyTag);
 
     if(emptyAction == "forceemptyinclude")
@@ -80,29 +83,53 @@ TagRenamerOptions::TagRenamerOptions(const CategoryID &category)
     else if(emptyAction == "usereplacementvalue")
         setEmptyAction(UseReplacementValue);
 
-    setEmptyText(config.readEntry(QString("%1EmptyText").arg(typeKey)));
-    setTrackWidth(config.readUnsignedNumEntry(QString("%1TrackWidth").arg(typeKey)));
-    setDisabled(config.readBoolEntry(QString("%1Disabled").arg(typeKey), disabled));
+    setEmptyText(config.readEntry(QString("%1EmptyText").arg(typeKey), QString()));
+    setTrackWidth(config.readEntry(QString("%1TrackWidth").arg(typeKey), 0));
+    setDisabled(config.readEntry(QString("%1Disabled").arg(typeKey), disabled));
 }
 
 QString TagRenamerOptions::tagTypeText(TagType type, bool translate)
 {
-    // These must be declared in the same order that they are defined in
-    // the TagType enum in test.h.  We can dynamically translate these strings,
-    // so make sure that I18N_NOOP() is used instead of i18n().
+    const char *msg = 0, *context = 0;
 
-    const char *tags[] = {
-        I18N_NOOP("Title"), I18N_NOOP("Artist"), I18N_NOOP("Album"),
-        I18N_NOOP("Track"), I18N_NOOP("Genre"), I18N_NOOP("Year")
-    };
+    switch(type) {
+        case Title:
+            msg = I18N_NOOP2("song title", "Title");
+            context = "song title";
+        break;
 
-    if(type < StartTag || type >= NumTypes) {
-        kdWarning() << "I don't know what category we're looking up, this is a problem." << endl;
-        kdWarning() << "The category ID is " << (unsigned) type << endl;
-        return translate ? i18n("Unknown") : "Unknown";
+        case Artist:
+            msg = I18N_NOOP("Artist");
+        break;
+
+        case Album:
+            msg = I18N_NOOP("Album");
+        break;
+
+        case Track:
+            msg = I18N_NOOP2("cd track number", "Track");
+            context = "cd track number";
+        break;
+
+        case Genre:
+            msg = I18N_NOOP("Genre");
+        break;
+
+        case Year:
+            msg = I18N_NOOP("Year");
+        break;
+
+        default:
+            kWarning() << "I don't know what category we're looking up, this is a problem.";
+            kWarning() << "The category ID is " << (unsigned) type;
+            msg = I18N_NOOP2("unknown renamer category", "Unknown");
+            context = "unknown renamer category";
     }
 
-    return translate ? i18n(tags[type]) : tags[type];
+    if(translate)
+        return context ? i18n(msg) : i18nc(context, msg);
+    else
+        return msg;
 }
 
 void TagRenamerOptions::saveConfig(unsigned categoryNum) const
@@ -153,4 +180,4 @@ TagType TagRenamerOptions::tagFromCategoryText(const QString &text)
     return Unknown;
 }
 
-// vim: set et ts=4 sw=4:
+// vim: set et sw=4 tw=0 sta:
