@@ -71,8 +71,9 @@ KMixWindow::KMixWindow(bool invisible)
 //   m_isVisible (false),    // initialize, as we don't trigger a hideEvent()
 //   m_visibilityUpdateAllowed( true ),
    m_multiDriverMode (false), // -<- I never-ever want the multi-drivermode to be activated by accident
-   m_dockWidget()
-   , _dockAreaPopup(0)
+   m_dockWidget(),
+   m_dontSetDefaultCardOnStart (false),
+   _dockAreaPopup(0)
 {
     setObjectName("KMixWindow");
     // disable delete-on-close because KMix might just sit in the background waiting for cards to be plugged in
@@ -214,17 +215,23 @@ bool KMixWindow::updateDocking()
       return false;
    }
 
-    // create dock widget and the corresponding popup
-    /* A GUIProfile does not make sense for the DockAreaPopup => Using (GUIProfile*)0 */
+   // create dock widget and the corresponding popup
+   /* A GUIProfile does not make sense for the DockAreaPopup => Using (GUIProfile*)0 */
    QWidget* referenceWidgetForSystray = this;
-     if ( m_volumeWidget ) {
-      _dockAreaPopup = new ViewDockAreaPopup(0, "dockArea", Mixer::getGlobalMasterMixer(), 0, (GUIProfile*)0, this);
+   if ( m_volumeWidget ) {
+      KMenu *volMenu = new KMenu(this);
+      _dockAreaPopup = new ViewDockAreaPopup(volMenu, "dockArea", Mixer::getGlobalMasterMixer(), 0, (GUIProfile*)0, this);
       _dockAreaPopup->createDeviceWidgets();
-      referenceWidgetForSystray = _dockAreaPopup;
-    }
-    m_dockWidget = new KMixDockWidget( this, referenceWidgetForSystray, _dockAreaPopup );
-    m_dockWidget->show();
-    return true;
+
+      QWidgetAction *volWA = new QWidgetAction(volMenu);
+      volWA->setDefaultWidget(_dockAreaPopup);
+      volMenu->addAction(volWA);
+      referenceWidgetForSystray = volMenu;
+   }
+   m_dockWidget = new KMixDockWidget( this, referenceWidgetForSystray, _dockAreaPopup );
+   //m_dockWidget->show();
+   connect(m_dockWidget, SIGNAL(newMasterSelected()), SLOT(saveConfig()) );
+   return true;
 }
 
 void KMixWindow::saveConfig()
@@ -266,7 +273,7 @@ void KMixWindow::saveBaseConfig()
    }
    QString mixerIgnoreExpression = MixerToolBox::instance()->mixerIgnoreExpression();
    config.writeEntry( "MixerIgnoreExpression", mixerIgnoreExpression );
-   
+
    // @todo basically this should be moved in the views later (e.g. KDE4.2 ?)
    if ( m_toplevelOrientation  == Qt::Horizontal )
       config.writeEntry( "Orientation","Horizontal" );
@@ -483,7 +490,7 @@ void KMixWindow::unplugged( const QString& udi)
                     Mixer::setGlobalMaster( ((Mixer::mixers())[0])->id(), localMaster);
                     
                     QString text;
-                    text = i18n("The soundcard containing the master device was unplugged. Changing to control %1 on card %2", 
+                    text = i18n("The soundcard containing the master device was unplugged. Changing to control %1 on card %2.", 
                             ((Mixer::mixers())[0])->getLocalMasterMD()->readableName(),
                             ((Mixer::mixers())[0])->readableName()
                                 );
@@ -607,7 +614,9 @@ void KMixWindow::hideOrClose ( )
 void KMixWindow::increaseOrDecreaseVolume(bool increase)
 {
   Mixer* mixer = Mixer::getGlobalMasterMixer(); // only needed for the awkward construct below
+  if ( mixer == 0 ) return; // e.g. when no soundcard is available
   MixDevice *md = Mixer::getGlobalMasterMD();
+  if ( md == 0 ) return; // shouldn't happen, but lets play safe
   md->setMuted(false);
   if (increase)
     mixer->increaseVolume(md->id());    // this is awkward. Better move the increaseVolume impl to the Volume class.
@@ -660,7 +669,9 @@ void KMixWindow::slotHideVolumeDisplay()
 void KMixWindow::slotMute()
 {
   Mixer* mixer = Mixer::getGlobalMasterMixer();
+  if ( mixer == 0 ) return; // e.g. when no soundcard is available
   MixDevice *md = Mixer::getGlobalMasterMD();
+  if ( md == 0 ) return; // shouldn't happen, but lets play safe
   mixer->toggleMute(md->id());
   showVolumeDisplay();
 }
