@@ -13,14 +13,17 @@
 
   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
-  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-  Boston, MA 02111-1307, USA.
+  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+  Boston, MA 02110-1301, USA.
 */
 
 #include "asyncsmtpsubmit.h"
+
 #include "cdinfo.h"
-#include <qdatastream.h>
+
+
 #include <kdebug.h>
+#include <kio/job.h>
 
 namespace KCDDB
 {
@@ -36,17 +39,17 @@ namespace KCDDB
 
   }
 
-  CDDB::Result AsyncSMTPSubmit::runJob(KIO::Job* job)
+  Result AsyncSMTPSubmit::runJob(KIO::Job* job)
   {
-    connect( job, SIGNAL( result( KIO::Job* ) ),
-               this, SLOT(slotDone( KIO::Job* ) ) ); 
+    connect( job, SIGNAL( result( KJob* ) ),
+               this, SLOT(slotDone( KJob* ) ) );
 
     return Success;
   }
 
-  void AsyncSMTPSubmit::slotDone( KIO::Job* job )
+  void AsyncSMTPSubmit::slotDone( KJob* job )
   {
-      kdDebug(60010) << k_funcinfo << endl;
+      kDebug(60010) ;
       if ( job->error()==0 )
         emit finished( Success );
       else
